/*
 * Copyright (C) 2003 - 2007 NetXen, Inc.
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * The full GNU General Public License is included in this distribution
 * in the file called LICENSE.
 * 
 * Contact Information:
 *    licensing@netxen.com
 * NetXen, Inc.
 * 3965 Freedom Circle, Fourth floor,
 * Santa Clara, CA 95054
 */
#include <linux/netdevice.h>
#include <linux/delay.h>

#include "queue.h"
#include "unm_nic.h"
#include "unm_nic_hw.h"
#include "nic_cmn.h"
#include "nic_phan_reg.h"
#ifdef __VMKERNEL_MODULE__
#include <linux/ethtool.h>
#endif /* __VMKERNEL_MODULE__ */

long
unm_nic_enable_phy_interrupts (unm_adapter *adapter)
{
        switch (adapter->ahw.board_type) {
        case UNM_NIC_GBE:
                return unm_niu_gbe_enable_phy_interrupts (adapter);

        case UNM_NIC_XGBE:
                return unm_niu_xg_enable_phy_interrupts (adapter);

        default:
                printk(KERN_ERR"%s: Unknown board type\n", unm_nic_driver_name);
		return -1;
        }
}

long
unm_nic_disable_phy_interrupts (unm_adapter *adapter)
{
        switch (adapter->ahw.board_type) {
        case UNM_NIC_GBE:
                return unm_niu_gbe_disable_phy_interrupts (adapter);

        case UNM_NIC_XGBE:
                return unm_niu_xg_disable_phy_interrupts (adapter);

        default:
                printk(KERN_ERR"%s: Unknown board type\n", unm_nic_driver_name);
		return -1;
        }
}

long
unm_nic_clear_phy_interrupts (unm_adapter *adapter)
{
        switch (adapter->ahw.board_type) {
        case UNM_NIC_GBE:
                return unm_niu_gbe_clear_phy_interrupts (adapter);

        case UNM_NIC_XGBE:
                return unm_niu_xg_disable_phy_interrupts (adapter);

        default:
                printk(KERN_ERR"%s: Unknown board type\n", unm_nic_driver_name);
		return -1;
        }
}

void
unm_indicate_link_status(unm_adapter *adapter, u32 link)
{
        struct net_device        *netdev = adapter->netdev;

        if(link) {
                if (netdev->flags & IFF_UP) {
                        netif_carrier_on(netdev);
                        netif_wake_queue(netdev);
                }
        } else {
                netif_carrier_off(netdev);
                netif_stop_queue(netdev);
#ifdef __VMKERNEL_MODULE__
		vmk_net_set_dev_link_state(netdev,NETDEV_LINK_DOWN);
#endif
        }
}

void
unm_nic_isr_other(struct unm_adapter_s *adapter)
{
	u32 portno = adapter->portnum;
	u32 val, linkup, qg_linksup;

	read_lock(&adapter->adapter_lock);
	val = UNM_NIC_PCI_READ_32(CRB_NORMALIZE(adapter, CRB_XG_STATE));
	read_unlock(&adapter->adapter_lock);

	val = val >> physical_port[adapter->portnum];
	if (val == adapter->ahw.qg_linksup)
		return;

	qg_linksup = adapter->ahw.qg_linksup;
	adapter->ahw.qg_linksup = val;
	DPRINTK(1, INFO, "%s: link update 0x%08x\n", unm_nic_driver_name, val);

	linkup = val & 1;
	if (linkup != (qg_linksup & 1)) {
		printk(KERN_INFO "%s: PORT %d link %s\n", unm_nic_driver_name,
				portno, ((linkup == 0) ? "down" : "up"));
		unm_indicate_link_status(adapter, linkup);
		if (linkup) {
			unm_nic_set_link_parameters(adapter);
#ifdef __VMKERNEL_MODULE__
			vmk_net_set_dev_link_state(adapter->netdev, NETDEV_LINK_UP);
			vmk_net_set_dev_speed(adapter->netdev, adapter->link_speed);
			vmk_net_set_dev_duplex(adapter->netdev, adapter->link_duplex);
#endif
		}

	}
	//adapter->stats.otherints++;
}

/*
 * return 1 if link is ok, 0 otherwise
 */
int
unm_link_ok(struct unm_adapter_s *adapter)
{
        switch (adapter->ahw.board_type) {
        case UNM_NIC_GBE:
                return ((adapter->ahw.qg_linksup) & 1);

        case UNM_NIC_XGBE:
                return ((adapter->ahw.xg_linkup) & 1);

        default:
                printk(KERN_ERR"%s: Function: %s, Unknown board type\n",
                        unm_nic_driver_name, __FUNCTION__);
                break;
        }

        return 0;
}
void
unm_nic_handle_phy_intr(struct unm_adapter_s *adapter)
{
        uint32_t  val, val1;

        switch (adapter->ahw.board_type) {
        case UNM_NIC_GBE:
                unm_nic_isr_other(adapter);
                break;
        case UNM_NIC_XGBE:
                {
                struct net_device  *netdev;

                /* WINDOW = 1 */
                //unm_nic_read_crb_w1(adapter, CRB_XG_STATE, &val1);
		read_lock(&adapter->adapter_lock);
                val = UNM_NIC_PCI_READ_32(
                                        CRB_NORMALIZE(adapter, CRB_XG_STATE));
		read_unlock(&adapter->adapter_lock);		
		netdev = adapter->netdev;
                val >>= (physical_port[adapter->portnum] * 8);
		val1 = val & 0xff;
		if ((adapter->ahw.xg_linkup & 1) && 
				(val1 != XG_LINK_UP)) {
			printk(KERN_ERR"%s: %s NIC Link is down\n",
					unm_nic_driver_name, netdev->name);
			unm_indicate_link_status(adapter, 0);
			adapter->ahw.xg_linkup &= ~1UL;
			/* read twice to clear sticky bits */
			/* WINDOW = 0 */
			/*unm_nic_read_crb_w0(adapter, UNM_NIU_XG_STATUS,
			  &val1);
			  unm_nic_read_crb_w0(adapter, UNM_NIU_XG_STATUS,
			  &val1);*/

		} else if(!(adapter->ahw.xg_linkup & 1) && 
				(val1 == XG_LINK_UP)) {
			printk(KERN_INFO"%s: %s NIC Link is up\n",
					unm_nic_driver_name, netdev->name);
			unm_indicate_link_status(adapter, 1);
			adapter->ahw.xg_linkup |= 1;
#ifdef __VMKERNEL_MODULE__
			vmk_net_set_dev_link_state(netdev,NETDEV_LINK_UP);
			vmk_net_set_dev_speed(netdev,SPEED_10000);
			vmk_net_set_dev_duplex(netdev,DUPLEX_FULL);
#endif
		}

		}
		break;

	default:
		printk(KERN_ERR"%s ISR: Unknown board type\n",
				unm_nic_driver_name);
	}

        return;
}
